Package: multtest
Title: Resampling-based multiple hypothesis testing
Version: 2.68.0
Author: Katherine S. Pollard, Houston N. Gilbert, Yongchao Ge, Sandra
        Taylor, Sandrine Dudoit
Description: Non-parametric bootstrap and permutation resampling-based
        multiple testing procedures (including empirical Bayes methods)
        for controlling the family-wise error rate (FWER), generalized
        family-wise error rate (gFWER), tail probability of the
        proportion of false positives (TPPFP), and false discovery rate
        (FDR).  Several choices of bootstrap-based null distribution
        are implemented (centered, centered and scaled,
        quantile-transformed). Single-step and step-wise methods are
        available. Tests based on a variety of t- and F-statistics
        (including t-statistics based on regression parameters from
        linear and survival models as well as those based on
        correlation parameters) are included.  When probing hypotheses
        with t-statistics, users may also select a potentially faster
        null distribution which is multivariate normal with mean zero
        and variance covariance matrix derived from the vector
        influence function.  Results are reported in terms of adjusted
        p-values, confidence regions and test statistic cutoffs. The
        procedures are directly applicable to identifying
        differentially expressed genes in DNA microarray experiments.
Maintainer: Katherine S. Pollard <katherine.pollard@gladstone.ucsf.edu>
Depends: R (>= 2.10), methods, BiocGenerics, Biobase
Imports: survival, MASS, stats4
Suggests: snow
License: LGPL
LazyLoad: yes
biocViews: Microarray, DifferentialExpression, MultipleComparison
git_url: https://git.bioconductor.org/packages/multtest
git_branch: RELEASE_3_23
git_last_commit: ea6e4ff
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 23:30:13 UTC; biocbuild
