useDynLib(multtest)
useDynLib(multtest, VScount=VScount)

import(methods, BiocGenerics, Biobase)

importFrom(graphics, plot)
importFrom(stats, update)

importFrom(survival, is.Surv)
importFrom(survival, coxph.control)
importFrom(survival, coxph.fit)

importFrom(MASS, mvrnorm)
importFrom(MASS, rlm)

importFrom("grDevices", "dev.interactive", "xy.coords")
  
importFrom("graphics", "matplot", "mtext", "par", "points", "rect",
             "segments", "strheight", "strwidth", "text", "title",
             "xinch", "yinch")
  
importFrom("stats", "approxfun", "cor", "cov", "cov2cor","lm.fit",
             "lm.wfit", "quantile", "rbinom", "rf", "rnorm", "rt",
           "runif")

  
importFrom("utils", "combn")

importMethodsFrom(stats4, plot, summary)

#snow does not have a NAMESPACE
#importFrom(snow, clusterApply)
#importFrom(snow, LBclusterApply)
#importFrom(snow, clusterEvalQ)
#importFrom(snow, makeCluster)
#importFrom(snow, stopCluster)


exportClasses(MTP, EBMTP)

exportMethods(as.list, plot, summary, update, EBupdate, "[")

export(MTP, EBMTP, as.list, mt.maxT, mt.minP, mt.plot, mt.rawp2adjp,
  mt.reject, mt.sample.label, mt.sample.rawp, mt.sample.teststat,
  mt.teststat, mt.teststat.num.denum, meanX, diffmeanX, FX, blockFX, 
  twowayFX, lmX, lmY, coxY, get.Tn, boot.null, boot.resample, 
  center.only, center.scale, quant.trans, fwer2gfwer, fwer2tppfp, 
  fwer2fdr, get.index, ss.maxT, ss.minP, sd.maxT, sd.minP, wapply, 
  corr.Tn, corr.null, IC.Cor.NA, IC.CorXW.NA, insert.NA, diffs.1.N, 
  marg.samp, tQuantTrans, G.VS, ABH.h0, dens.est, Hsets, VScount,
  mtp2ebmtp, ebmtp2mtp)
  
S3method(print, MTP)
S3method(print, EBMTP)
